@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013,2014,2015,2016,2017,2018
+;;  Copyright (C) 2013-2023
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -29,7 +29,8 @@
             get-conf
             current-conf-file
             init-database-config
-            current-myhost))
+            current-myhost
+            default-conf-values))
 
 (define server-info artanis-version)
 (define *default-conf-file* "/etc/artanis/artanis.conf")
@@ -118,6 +119,9 @@ db.pool = increase | fixed")
     ((db encodeparams)
      #f
      "Whether to encode params automatically.
+(params rc \"your_key\") will be encoded by uri-encode.
+The username and passwd in :auth will be encoded by uri-encode.
+NOTE: It's your duty to call uri-decode to get proper value.
 NOTE: If you enable db.encodeparams then it's better to decode the related value
 twice in the client-side, since some requests may be sent from browsers, and
 they're already encoded.
@@ -214,7 +218,7 @@ server.bufsize = <integer>")
     ;;       SO_REUSEPORT (since 3.9)
     ;;       Allows mutiple servers to listen to the same socket port, say 8080.
     ((server multi)
-     #t
+     #f
      "This is the most significant feature of Ragnarok server core.
 Please remember that there're no threads in GNU Artanis.
 All the tasks are based on delimited continuations, this kind of design is the
@@ -241,6 +245,21 @@ server.pub = <string>")
      "Whether to use Linux specified sendfile interface.
 server.sendfile = <boolean>")
 
+    ((server mmapped)
+     #f
+     "Whether to use POSIX specific mmap for file I/O.
+server.mmapped = <boolean>")
+
+    ((server allowedmethods)
+     (HEAD GET POST PUT)
+     "The allowed HTTP methods.
+server.allowedmethods = <methods-list>")
+
+    ((server jsmanifest)
+     "pub"
+     "The path to find \"manifest.json\".
+server.jsmanifest = <string>")
+
     ((websocket maxpayload)
      ,(1- (ash 1 64))
      "The maximum payload size of WebSocket request in bytes. If it exceeds, then it
@@ -274,8 +293,8 @@ websocket.timeout = <integer>")
     ((host name)
      #f
      "If disabled, you will have to use the IP address to connect instead of the
-hostname. e.g. `host.addr = 127.0.0.1'.
-host.name = enable | disable | <boolean>")
+hostname. e.g. `host.name = lambdachip.com'.
+host.name = <string> | disable")
 
     ((host addr)
      "127.0.0.1"
@@ -292,12 +311,23 @@ host.port = <integer>")
      "Specify the protocol family.
 host.family = ipv4 | ipv6")
 
+    ((host detectpath)
+     #f
+     "Detect host or domain-name from specified path. This is useful when you run GNU Artanis inside a container.
+host.detectpath = <PATH>|<boolean>")
+
     ;; for session namespace
     ((session path)
      "session"
      "Specify the session files path. Change according to your session engine.
 session.path = <PATH>")
 
+    ((session hijackcheck)
+     "false"
+     "Check the valid session, if it appears in multiple client IP, then fail. This may
+cause problems when your hosting is behind CDN, you may need to disable it if so.
+session.hijackcheck = <boolean>")
+
     ((session backend)
      simple
      "Specify session engine. Here're supported backends:
@@ -323,7 +353,9 @@ upload.path = <PATH>")
 upload.size = <interger>")
 
     ;; for mail namespace
-    ;; ((mail sender) "/usr/sbin/sendmail")
+    ((mail sender) "/usr/bin/msmtp"
+     "The command called by Sendmail module. It's strongly recommended to use `msmtp' to replace `sendmail'.
+mail.sender = <string>")
 
     ((cache maxage)
      3600
@@ -345,7 +377,14 @@ debug.enable = <boolean>")
      "The paths that needs to be monitored in debug-mode.
 This will take advantage of `inotify' in GNU/Linux kernel.
 NOTE: We may support GNU/Hurd as well, with its file monitor mechanism, in the future.
-debug.monitor = <PATHs>")))
+debug.monitor = <PATHs>")
+    ((cookie expires)
+     3600
+     "Cookie expiration time in seconds.
+       1 hour is 3600
+       6 hours 21600
+       1 month 2592000
+ cookie.expires = <integer>")))
 
 ;; Init all fields with default values
 (for-each (lambda (x) (conf-set! (car x) (cadr x))) (default-conf-values))
@@ -361,6 +400,11 @@ debug.monitor = <PATHs>")))
          (string->symbol (string-trim-both e)))
        (filter (lambda (s) (not (string-null? s))) (string-split x #\,))))
 
+(define-syntax-rule (->string-list x)
+  (map (lambda (e)
+         (string-trim-both e))
+       (filter (lambda (s) (not (string-null? s))) (string-split x #\,))))
+
 (define-syntax-rule (->none/str x)
   (case (string->symbol (string-downcase x))
     ((none null #{}#) "")
@@ -368,7 +412,7 @@ debug.monitor = <PATHs>")))
 
 (define-syntax-rule (->none/boolean x)
   (case (string->symbol (string-downcase x))
-    ((false no none null) #f)
+    ((false no none null disable) #f)
     (else x)))
 
 (define-syntax-rule (->integer x)
@@ -394,12 +438,23 @@ debug.monitor = <PATHs>")))
       #f
       x))
 
+(define-syntax-rule (->string/boolean x)
+  (case (string->symbol (string-downcase x))
+    ((true on yes enable) #t)
+    ((false off no disable) #f)
+    (else x)))
+
 (define-syntax-rule (->dbd x)
   (let ((d (->symbol x)))
     (if (eq? 'mariadb d)
         'mysql
         d)))
 
+(define-syntax-rule (->methods ml)
+  (map (lambda (m)
+         (string->symbol (string-upcase m)))
+       (->string-list ml)))
+
 (define *pool-modes* '(increase fixed))
 (define-syntax-rule (->pool-mode x)
   (let ((p (string->symbol x)))
@@ -444,6 +499,9 @@ debug.monitor = <PATHs>")))
     (('websocket websocket) (conf-set! '(server websocket) (->bool websocket)))
     (('pub pub) (conf-set! '(server pub) (basename pub)))
     (('sendfile v) (conf-set! '(server sendfile) (->bool v)))
+    (('mmapped v) (conf-set! '(server mmapped) (->bool v)))
+    (('allowedmethods ml) (conf-set! '(server allowedmethods) (->methods ml)))
+    (('jsmanifest jm) (conf-set! '(server jsmanifest) (->none/str jm)))
     (else (error parse-namespace-server "Config: Invalid item" item))))
 
 (define (parse-namespace-websocket item)
@@ -457,10 +515,11 @@ debug.monitor = <PATHs>")))
 
 (define (parse-namespace-host item)
   (match item
-    (('name name) (conf-set! '(host name) name))
+    (('name name) (conf-set! '(host name) (->none/boolean name)))
     (('family family) (conf-set! '(host family) (->symbol family)))
     (('addr addr) (conf-set! '(host addr) addr))
     (('port port) (conf-set! '(host port) (->integer port)))
+    (('detectpath path) (conf-set! '(host detectpath) (->string/boolean path)))
     (else (error parse-namespace-host "Config: Invalid item" item))))
 
 (define (parse-namespace-backend backend)
@@ -479,6 +538,7 @@ debug.monitor = <PATHs>")))
 (define (parse-namespace-session item)
   (match item
     (('path path) (conf-set! '(session path) path))
+    (('hijackcheck check) (conf-set! '(session hijackcheck) (->bool check)))
     (('backend backend) (conf-set! '(session backend) (parse-namespace-backend backend)))
     (else (error parse-namespace-session "Config: Invalid item" item))))
 
@@ -497,13 +557,18 @@ debug.monitor = <PATHs>")))
 (define (parse-namespace-cache item)
   (match item
     (('maxage maxage) (conf-set! '(cache maxage) (->integer maxage)))
-    (else (error parse-namespace-mail "Config: Invalid item" item))))
+    (else (error parse-namespace-cache "Config: Invalid item" item))))
 
 (define (parse-namespace-debug item)
   (match item
     (('enable enable) (conf-set! 'debug-mode (->bool enable)))
     (('monitor monitor) (conf-set! '(debug monitor) (->list monitor)))
-    (else (error parse-namespace-cache "Config: Invalid item" item))))
+    (else (error parse-namespace-debug "Config: Invalid item" item))))
+
+(define (parse-namespace-cookie item)
+  (match item
+    (('expires expires) (conf-set! '(cookie expires) (->integer expires)))
+    (else (error parse-namespace-cookie "Config: Invalid item" item))))
 
 (define (parse-config-item item)
   (match item
@@ -516,39 +581,40 @@ debug.monitor = <PATHs>")))
     (('mail rest ...) (parse-namespace-mail rest))
     (('cache rest ...) (parse-namespace-cache rest))
     (('debug rest ...) (parse-namespace-debug rest))
+    (('cookie rest ...) (parse-namespace-cookie rest))
     (((? string-null?)) #t) ; skip
     (else (error parse-config-item "Unsupported config namespace!" item))))
 
 (define (parse-line line)
   (call-with-input-string
-      line
-    (lambda (port)
-      (let lp((next (read-char port)) (key? #t) (word '()) (ret '()))
-        (cond
-         ((or (eof-object? next)
-              (char=? next #\#)) ; skip comment
-          (reverse (cons (list->string (reverse word)) ret)))
-         ((char-set-contains? char-set:whitespace next)
-          ;; skip all whitespaces
-          (lp (read-char port) key? word ret))
-         ((and key? (char=? next #\.))
-          ;; a namespace end
-          (lp (read-char port) key? '() (cons (list->symbol (reverse word)) ret)))
-         ((and key? (char=? next #\=))
-          ;; value start
-          (lp (read-char port) #f '() (cons (list->symbol (reverse word)) ret)))
-         ((not key?)
-          ;; store chars of value
-          (lp (read-char port) key? (cons next word) ret))
-         (else
-          ;; store chars of key
-          (lp (read-char port) key? (cons next word) ret)))))))
+   line
+   (lambda (port)
+     (let lp((next (read-char port)) (key? #t) (word '()) (ret '()))
+       (cond
+        ((or (eof-object? next)
+             (char=? next #\#)) ; skip comment
+         (reverse (cons (list->string (reverse word)) ret)))
+        ((char-set-contains? char-set:whitespace next)
+         ;; skip all whitespaces
+         (lp (read-char port) key? word ret))
+        ((and key? (char=? next #\.))
+         ;; a namespace end
+         (lp (read-char port) key? '() (cons (list->symbol (reverse word)) ret)))
+        ((and key? (char=? next #\=))
+         ;; value start
+         (lp (read-char port) #f '() (cons (list->symbol (reverse word)) ret)))
+        ((not key?)
+         ;; store chars of value
+         (lp (read-char port) key? (cons next word) ret))
+        (else
+         ;; store chars of key
+         (lp (read-char port) key? (cons next word) ret)))))))
 
 (define (init-inner-database-item)
   (define dbd (get-conf '(db dbd)))
   (define proto (get-conf '(db proto)))
   (define addr (get-conf '(db addr)))
-  (define sock (and=> (get-conf '(db socketfile)) (lambda (x) (not (eq? x 'disable)))))
+  (define sock (get-conf '(db socketfile)))
   (define dbname (get-conf '(db name)))
   (define username (get-conf '(db username)))
   (define passwd (get-conf '(db passwd)))
@@ -596,15 +662,37 @@ debug.monitor = <PATHs>")))
 ;;       art work -c ./my.conf
 ;; And init-server should be called automatically.
 (define current-conf-file (make-parameter #f))
-(define (current-myhost)
-  (format #f "http://~a:~a" (get-conf '(host addr)) (get-conf '(host port))))
+(define* (current-myhost #:key (for-header? #f))
+  (define (domain file)
+    (if (and file (file-exists? file))
+        (string-trim-both (call-with-input-file file read-line))
+        #f))
+  (define (get-domain-name)
+    (cond
+     ((domain (get-conf '(host detectpath))) => identity)
+     ((getenv "DOMAIN_NAME") => identity)
+     ((file-exists? "/proc/sys/kernel/domainname")
+      (let ((domain (string-trim-both
+                     (call-with-input-file "/proc/sys/kernel/domainname" read-line))))
+        (if (string=? "(none)" domain)
+            #f
+            domain)))
+     (else #f)))
+  (let* ((host (or (get-domain-name) (get-conf '(host name))))
+         (real-host (if host host (get-conf '(host addr))))
+         (port (get-conf '(host port))))
+    (if for-header?
+        (cons real-host #f)
+        (if host
+            (format #f "http://~a" real-host)
+            (format #f "http://~a:~a" real-host port)))))
 
 (define (init-config)
   (define conf-file (current-conf-file)) ; user specified config
   (define fp (open-input-file
               (cond
                ((and conf-file (file-exists? conf-file))
-                (format (artanis-current-output) "Loading conf/artanis.conf...")
+                (format (artanis-current-output) "Loading ~a..." conf-file)
                 conf-file)
                ((file-exists? *default-conf-file*)
                 (format (artanis-current-output) "Loading ~a..." *default-conf-file*)
