@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2014,2015,2016,2017,2018,2019,2020,2021
+;;  Copyright (C) 2014-2021,2024
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -40,6 +40,7 @@
   #:use-module (artanis third-party json)
   #:use-module (artanis third-party csv)
   #:use-module (artanis server scheduler)
+  #:use-module (artanis security nss)
   #:use-module (artanis irregex)
   #:use-module (artanis lpc)
   #:use-module (ice-9 match)
@@ -151,11 +152,11 @@
 (define (conn-maker yes? rule keys)
   (and yes?
        (case-lambda
-         ((rc) (DB-open rc))
-         ((rc sql)
-          (let ((conn (DB-open rc)))
-            (DB-query conn sql)
-            conn)))))
+        ((rc) (DB-open rc))
+        ((rc sql)
+         (let ((conn (DB-open rc)))
+           (DB-query conn sql)
+           conn)))))
 
 (define (raw-sql-maker sql rule keys)
   (lambda (rc mode)
@@ -277,10 +278,8 @@
       (`(spawn ,sid) (cut %spawn <> #:idname sid <...>))
       (`(spawn ,sid ,proc) (cut %spawn <> #:idname sid #:proc proc <...>))
       (else (throw 'artanis-err 500 session-maker "Invalid config mode: ~a" mode))))
-  (define (get-sid rc idname)
-    (any (lambda (c) (and=> (cookie-ref c idname) car)) (rc-cookie rc)))
   (define (check-it rc idname)
-    (let* ((sid (get-sid rc idname))
+    (let* ((sid (get-sid-from-client-cookie rc idname))
            (session (session-restore (or sid ""))))
       (case session
         ((expired)
@@ -288,7 +287,8 @@
          #f)
         ((not-found) #f)
         (else
-         (session-from-correct-client? session rc)))))
+         (when (get-conf '(session hijackcheck))
+           (session-from-correct-client? session rc))))))
   (lambda (rc cmd . args)
     (match cmd
       ('check (check-it rc "sid"))
@@ -298,11 +298,13 @@
       (`(check-and-spawn ,sid)
        (or (check-it rc sid) (apply spawn-handler rc args)))
       ('spawn (apply spawn-handler rc args))
+      ('get (session-restore (get-sid-from-client-cookie rc "sid")))
+      (`(get ,sid) (session-restore (get-sid-from-client-cookie rc sid)))
       ('drop
-       (session-destroy! (get-sid rc "sid"))
+       (session-destroy! (get-sid-from-client-cookie rc "sid"))
        (:cookies-remove! rc "sid"))
       (`(drop ,sid)
-       (session-destroy! (get-sid rc sid))
+       (session-destroy! (get-sid-from-client-cookie rc sid))
        (:cookies-remove! rc sid))
       (else (throw 'artanis-err 500 session-maker "Invalid call cmd: ~a" cmd)))))
 
@@ -314,10 +316,10 @@
         '()))
   (define (default-success-ret size-list filename-list)
     (call-with-output-string
-      (lambda (port)
-        (for-each (lambda (s f)
-                    (format port "<p>Upload succeeded! ~a: ~a bytes!</p>" s f))
-                  size-list filename-list))))
+     (lambda (port)
+       (for-each (lambda (s f)
+                   (format port "<p>Upload succeeded! ~a: ~a bytes!</p>" f s))
+                 size-list filename-list))))
   (define (default-no-file-ret) "<p>No uploaded files!</p>")
   (define* (store-the-bv rc #:key (uid 33) (gid 33) (path (get-conf '(upload path)))
                          (mode #o664) (path-mode #o775) (sync #f) (simple-ret? #t)
@@ -490,7 +492,9 @@
       (string=? p stored-pw)))
   (define (init-post rc)
     (and (rc-body rc)
-         (generate-kv-from-post-qstr (rc-body rc))))
+         (generate-kv-from-post-qstr
+          (rc-body rc)
+          #:no-evil? (get-conf '(db encodeparams)))))
   (define (default-hmac pw salt)
     ;; We still have sha384, sha512 as options, but I think sha256 is enough.
     (string->sha-256 (string-append pw salt)))
