@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2017,2018,2019
+;;  Copyright (C) 2017-2019,2024
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -21,12 +21,12 @@
   #:use-module (artanis utils)
   #:use-module (artanis env)
   #:use-module (artanis config)
-  #:use-module (artanis crypto base64)
   #:use-module (artanis server server-context)
   #:use-module (artanis server scheduler)
   #:use-module (artanis irregex)
   #:use-module (artanis websocket frame)
   #:use-module (artanis websocket protocols)
+  #:use-module (artanis security nss)
   #:use-module (ice-9 iconv)
   #:use-module (rnrs bytevectors)
   #:use-module ((rnrs) #:select (define-record-type))
@@ -78,7 +78,7 @@
   (let* ((realkey (string-append key *ws-magic*))
          (keyhash (string->sha-1 realkey))
          (keybv (list->u8vector (string->byteslist keyhash 2 16))))
-    (base64-encode keybv)))
+    (nss:base64-encode keybv)))
 
 (define (validate-websocket-request req client)
   (define (not-proper-websocket-version? version)
